<template>
    <div class="system-gas-container">
        <el-card shadow="hover">
            <div class="system-menu-search mb15">
                <el-form :inline="true" style="display: flex;align-items: flex-start;flex-wrap: wrap" >
                    <el-form-item label="日期：" >
                        <el-date-picker
                            v-model="state.tableData.listQuery.searchParams.time"
                            type="datetimerange"
                            format="YYYY-MM-DD HH:mm:ss"
                            range-separator="~"
                            start-placeholder="开始时间"
                            end-placeholder="结束时间"
                            @change = "chooseTime"
                        />
                    </el-form-item>
                    <el-form-item label="区域：">
                        <el-select
                            v-model="state.tableData.listQuery.searchParams.area"
                            class="w100"
                            style="max-width: 180px"
                            size="default"
                        >
                            <el-option v-for="item in state.tableData.areaList" :key="item.id" :label="item.name" :value="item.id"></el-option>
                        </el-select>
                    </el-form-item>
                    <el-form-item label="气体：">
                        <el-select
                            v-model="state.tableData.listQuery.searchParams.gas"
                            class="w100"
                            style="max-width: 180px"
                            size="default"
                        >
                            <el-option v-for="item in state.tableData.gasList" :key="item.id" :label="item.name" :value="item.id"></el-option>
                        </el-select>
                    </el-form-item>
                    <el-button size="default" type="primary" class="ml10" @click="search()">
                        <el-icon>
                            <ele-Search />
                        </el-icon>
                        查询
                    </el-button>
                    <el-button size="default" class="ml10" @click="reset()">
                        <el-icon>
                            <RefreshLeft />
                        </el-icon>
                        重置
                    </el-button>
                </el-form>
            </div>
            <div id="gasChart" style="height: 500px;width: auto"></div>
            <el-table :data="state.tableData.data" style="width: 100%">
                <el-table-column type="index" label="序号" width="80" />
                <el-table-column align="center" prop="time" label="采集时间"/>
                <el-table-column align="center" prop="regionName" label="区域名称"/>
                <el-table-column align="center" prop="windSpeed" label="风速"/>
                <el-table-column align="center" prop="windDirection" label="风向"/>
                <el-table-column align="center" prop="name" label="气体名称"/>
                <el-table-column align="center" prop="gasValue" label="气体浓度"/>
            </el-table>
            <br />
            <el-pagination
                @size-change="onHandleSizeChange"
                @current-change="onHandleCurrentChange"
                class="page-position"
                :pager-count="5"
                :page-sizes="[10, 20, 30]"
                v-model:current-page="state.tableData.listQuery.pageIndex"
                background
                v-model:page-size="state.tableData.listQuery.pageSize"
                layout="total, sizes, prev, pager, next, jumper"
                :total="state.tableData.total">
            </el-pagination>
            <br />
            <br />
        </el-card>
    </div>
</template>

<script setup lang="ts">
import {reactive, ref,onMounted} from "vue";
import * as echarts from "echarts";
import { ElMessage, ElMessageBox } from 'element-plus'
import {TableFluxState} from "/@/types/monitorData";
import moment from "moment/moment";
import { gasManageApi } from "/@/api/basicDataManage/gasManage";
import { areaManageApi } from "/@/api/basicDataManage/areaManage";
import { fluxDataApi } from "/@/api/monitorData/fluxData";

const state = reactive<TableFluxState>({
    tableData: {
        data: [],
        total: 0,
        loading: false,
        listQuery: {
            pageIndex: 1,
            pageSize: 10,
            searchParams:{
                time: [],
                gas: '',
                area: ''
            }
        },
        gasList: [],
        areaList: [],
    }
});

const chooseTime = (val:any) => {
    console.log("val",val)
    let sTime = Date.parse(new Date(val[0]));
    let eTime = Date.parse(new Date(val[1]));
    console.log(sTime, eTime)
    // 2.时间戳间隔 再加上一天
    const datadiff = eTime - sTime + 86400000;
    // 3.七天间隔
    const time = 7 * 24 * 60 * 60 * 1000;
    // 4.比较两个日期的远近，date2为最近的日期
    if (sTime > eTime) {
        return false;
    } else {
        if (datadiff > time) {
            ElMessage({
                type: 'error',
                message: '查询时间范围7天内',
            })
            getNowTime();
            return false;
        } else {
            console.log('七天内数据')
        }
    }
}

onMounted(
    () => {
        getNowTime();
        getArea();
        getAllGas();
    }
);

const getNowTime = () => {
    let isDate = new Date()
    let sTime = `${isDate.getFullYear()}-${isDate.getMonth() + 1}-${isDate.getDate()}`
    let eTime = `${isDate.getFullYear()}-${isDate.getMonth() + 1}-${isDate.getDate()}`
    sTime = `${sTime} 00:00:00`
    eTime = `${eTime} ` + moment().format('HH:mm:ss')
    state.tableData.listQuery.searchParams.time = [sTime ,eTime];
}

const dataZoomEnd = ref();
const xData = ref([] as any);
const yData = ref([]);
const areaChart = ref([]as any);
const initInfoData = async () => {
    //折线图
    const chartParam = {
        startTime: moment(state.tableData.listQuery.searchParams.time[0]).format('YYYY-MM-DD HH:mm:ss'),
        endTime: moment(state.tableData.listQuery.searchParams.time[1]).format('YYYY-MM-DD HH:mm:ss'),
        gasName: state.tableData.listQuery.searchParams.gas,
        areaId: state.tableData.listQuery.searchParams.area
    }
    let resChart = await fluxDataApi().getFluxLineChart(chartParam);
    if(resChart.data.code == 100) {
        yData.value = mapArray(resChart.data.data);
        areaChart.value = yData.value.map((item: any) => {
            return item.name
        });
        dataZoomEnd.value = xData.value.length > 25 ? 2 : 100;
        initCharts();
    }else {
        ElMessage({
            type: 'warning',
            message: resChart.data.msg
        });
    }

    //表格
    const pageParam = {
        pageIndex: state.tableData.listQuery.pageIndex,
        pageSize: state.tableData.listQuery.pageSize,
        searchParams: {
            startTime: moment(state.tableData.listQuery.searchParams.time[0]).format('YYYY-MM-DD HH:mm:ss'),
            endTime: moment(state.tableData.listQuery.searchParams.time[1]).format('YYYY-MM-DD HH:mm:ss'),
            gasName: state.tableData.listQuery.searchParams.gas,
            areaId: state.tableData.listQuery.searchParams.area
        }
    }
    let res = await fluxDataApi().getFluxLinePage(pageParam);
    if(res.data.code == 100) {
        state.tableData.data = res.data.data;
        state.tableData.total = res.data.total;
        state.tableData.listQuery.pageIndex = res.data.pageIndex;
        state.tableData.listQuery.pageSize = res.data.pageSize;
    }else {
        ElMessage({
            type: 'warning',
            message: res.data.msg
        });
    }
};

const labelOption = {
    rotate: 90,
    align: 'left',
    verticalAlign: 'middle',
    position: 'insideBottom',
    distance: 15,
};
const mapArray = (target: any) => {
    let obj = {}as any;
    let result = [];
    let finalResult = []as any;
    target.map((item: any) => {
        let key = item.regionName;
        //如果有这个key，就push一个对象
        if (obj[key]) {
            obj[key].push(item);
        } else {
            obj[key] = [item];
        }
    });
    //根据obj中有几个对象，挨个push进去，组成最后的数组
    for (const key in obj) {
        const element = obj[key];
        result.push(element);
    }
    xData.value = result[0].map((item:any) => {
        return item.time;
    })
    result.forEach(item => {
        let obj = {
            name: '',
            data: [],
            type: 'bar',
            label: labelOption,
            emphasis:{
                focus: 'series'
            }
        };
       const data = item.map((i:any) => {
            return i.gasValue
        });
       obj.name = item[0].regionName;
       obj.data = data;
        finalResult.push(obj);
    })
    return finalResult;
}


const getArea = async () => {
    const param = {
        pageIndex: 1,
        pageSize: 999,
        searchParams:{
            name: ''
        }
    }
    let res = await areaManageApi().getAreaPage(param);
    if(res.data.code == 100) {
        state.tableData.areaList = res.data.data;
        // state.tableData.listQuery.searchParams.area = res.data.data[0].id;
    }else {
        ElMessage({
            type: 'warning',
            message: res.data.msg
        });
    }
}
const getAllGas = async () => {
    let res = await gasManageApi().getGas({});
    if(res.data.code == 100) {
        state.tableData.gasList = res.data.data;
        state.tableData.listQuery.searchParams.gas = res.data.data[0].id;
        await initInfoData();
    }else{
        ElMessage({
            type: 'warning',
            message: res.data.msg
        });
    }
};

const onHandleSizeChange = (val: number) => {
    state.tableData.listQuery.pageSize = val;
    initInfoData();
};
// 分页改变
const onHandleCurrentChange = (val: number) => {
    state.tableData.listQuery.pageIndex = val;
    initInfoData();
};
const search = () => {
    initInfoData();
    console.log("22",state.tableData.listQuery.searchParams)
}
const reset = () => {
    getNowTime();
    state.tableData.listQuery.searchParams.area = '';
    state.tableData.listQuery.searchParams.gas = state.tableData.gasList[0].id;
    state.tableData.listQuery.pageIndex = 1;
    initInfoData();
}


const initCharts = () => {
    const myChart = echarts.init(document.getElementById('gasChart'));
    // 指定图表的配置项和数据
    const option = {
        tooltip: {
            trigger: 'axis',
            axisPointer: {
                type: 'shadow'
            }
        },
        legend: {
            data: areaChart.value
        },
        toolbox: {
            show: true,
            orient: 'vertical',
            left: 'right',
            top: 'center',
            feature: {
                mark: { show: true },
                dataView: { show: true, readOnly: false },
                magicType: { show: true, type: ['line', 'bar', 'stack'] },
                restore: { show: true },
                saveAsImage: { show: true }
            }
        },
        xAxis: [
            {
                type: 'category',
                axisTick: { show: false },
                data: xData.value
            }
        ],
        yAxis: [
            {
                type: 'value'
            }
        ],
        series: yData.value,

        dataZoom: [
            {
                type: 'slider',
                show: dataZoomEnd.value == 100 ? false : true,
                realtime: true,
                start: 0,
                end: dataZoomEnd.value
            },
            {
                type: 'inside',
                realtime: true,
                start: 0,
                end: dataZoomEnd.value
            },
        ]
    };
    // 使用刚指定的配置项和数据显示图表。
    myChart.setOption(option,true);
    //自适应宽度
    window.addEventListener('resize', function () {
        myChart.resize();
    })
}
</script>
<style scoped lang="scss">
.yellow{
    width: 80px;
    height: 30px;
    background-color: rgb(255,223,37);
    line-height: 30px;
    color: white;
    box-shadow: 4px 4px 4px rgba(0, 0, 0, 0.2);
    padding: 3px
}
.red{
    width: 80px;
    height: 30px;
    background-color: rgb(239,90,161);
    line-height: 30px;
    color: white;
    box-shadow: 4px 4px 4px rgba(0, 0, 0, 0.2);
    padding: 3px
}
</style>